"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ember_cli_entities_1 = require("./lib/utilities/ember-cli-entities");
const path_1 = __importDefault(require("path"));
const broccoli_funnel_1 = __importDefault(require("broccoli-funnel"));
const sass_compiler_1 = __importDefault(require("./sass-compiler"));
const broccoli_merge_trees_1 = __importDefault(require("broccoli-merge-trees"));
const glob_1 = require("glob");
const broccoli_file_creator_1 = __importDefault(require("broccoli-file-creator"));
class SASSPlugin {
    constructor(optionsFn) {
        this.name = "fast-sass";
        this.optionsFn = optionsFn;
        this.ext = ["scss", "sass"];
    }
}
SASSPlugin.prototype.toTree = function (tree, inputPath, _outputPath, inputOptions) {
    var options = Object.assign({}, this.optionsFn(), inputOptions);
    if (options.autoIncludeComponentCSS) {
        const componentSCSSFiles = glob_1.glob.sync(path_1.default.join(options.appRoot, "/components/**/*.scss"));
        const result = componentSCSSFiles
            .map((filename) => `@import '${filename}';`)
            .join("\n");
        const importTree = broccoli_file_creator_1.default("/app/styles/_pod-styles.scss", result);
        tree = broccoli_merge_trees_1.default([tree, importTree]);
    }
    var inputTrees;
    if (options.onlyIncluded) {
        inputTrees = [
            new broccoli_funnel_1.default(tree, {
                include: ["app/styles/**/*"],
                annotation: "Funnel (styles)",
            }),
        ];
    }
    else {
        inputTrees = [
            new broccoli_funnel_1.default(tree, {
                include: ["**/*.scss", "**/*.css"],
            }),
        ];
    }
    if (options.includePaths) {
        inputTrees = inputTrees.concat(options.includePaths);
    }
    const ext = options.ext || "scss";
    var paths = options.outputPaths;
    var trees = Object.keys(paths).map(function (file) {
        var input = path_1.default.join(inputPath, `${file}.${ext}`);
        var output = paths[file];
        return new sass_compiler_1.default(inputTrees, input, output, options);
    });
    return broccoli_merge_trees_1.default(trees);
};
exports.default = ember_cli_entities_1.addon({
    name: "fast-sass",
    included(app) {
        this._super.included.apply(this, arguments);
        this.app = app;
        app.registry.add("css", new SASSPlugin(this.sassOptions.bind(this)));
    },
    sassOptions() {
        var _a, _b, _c, _d, _e;
        const env = process.env.EMBER_ENV;
        const app = this.app;
        const parent = this.parent;
        const hostApp = this._findHost && this._findHost();
        const options = (app && app.options && app.options.sassOptions) ||
            (parent && parent.options && parent.options.sassOptions) ||
            (hostApp && hostApp.options && hostApp.options.sassOptions) ||
            {};
        if (options.sourceMap === undefined && env == "development") {
            options.sourceMap = true;
        }
        if (options.sourceMap || options.sourceMapEmbed) {
            // we need to embed the sourcesContent in the source map until libsass has better support for broccoli-sass
            options.sourceMapContents = true;
        }
        options.outputFile = options.outputFile || this.project.name() + ".css";
        options.sourceMapRoot = path_1.default.join(this.project.root, "app/styles");
        if (typeof ((_b = (_a = this.app) === null || _a === void 0 ? void 0 : _a.trees) === null || _b === void 0 ? void 0 : _b.app) == "string") {
            options.appRoot = this.app.trees.app;
        }
        else {
            options.appRoot = (_e = (_d = (_c = this.app) === null || _c === void 0 ? void 0 : _c.trees) === null || _d === void 0 ? void 0 : _d.app) === null || _e === void 0 ? void 0 : _e._directoryPath;
        }
        return options;
    },
});
